@@ -24,7 +24,7 @@ class WURFLMiddleWare(object):
             device = self.devices.select_ua(environ['HTTP_USER_AGENT'],
                                             self.search_algorithm)
         except DeviceNotFound:
-            device = devices.select_id('generic')
+            device = devices.select_id(u'generic')
         environ['wurfl.device'] = device
         return self.application(environ, start_response)
 
@@ -35,7 +35,7 @@ def WURFLMiddleWareClosure(application, devices, search_algorithm):
             device = devices.select_ua(environ['HTTP_USER_AGENT'],
                                                search_algorithm)
         except DeviceNotFound:
-            device = devices.select_id('generic')
+            device = devices.select_id(u'generic')
         environ['wurfl.device'] = device
         return application(environ, start_response)
     return middle_ware
