@@ -1,5 +1,5 @@
 # pywurfl - Wireless Universal Resource File Tools in Python
-# Copyright (C) 2006-2010 Armand Lynch
+# Copyright (C) 2006-2011 Armand Lynch
 #
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -30,10 +30,10 @@ The original Java code is GPLd and Copyright (c) WURFL-Pro srl
 """
 
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
-__copyright__ = "Copyright 2010, Armand Lynch"
+__copyright__ = "Copyright 2011, Armand Lynch"
 __license__ = "LGPL"
 __url__ = "http://celljam.net/"
-__version__ = "1.2"
+__version__ = "1.2.1"
 
 import re
 
@@ -622,6 +622,9 @@ class SagemMatcher(AbstractMatcher):
 
 
 class SamsungMatcher(AbstractMatcher):
+    SAMSUNGS = [u"SEC-", u"SAMSUNG-", u"SCH", u"Samsung", u"SPH", u"SGH",
+                u"SAMSUNG/"]
+
     def can_handle(self, user_agent):
         return (u"Samsung/SGH" in user_agent or
                 u"Samsung" in user_agent or
@@ -632,20 +635,15 @@ class SamsungMatcher(AbstractMatcher):
                 user_agent.startswith(u"SCH"))
 
     def find_matching_ua(self, user_agent):
-        if (user_agent.startswith(u"SEC-") or
-            user_agent.startswith(u"SAMSUNG-") or
-            user_agent.startswith(u"SCH")):
-            tolerance = first_slash(user_agent)
-        elif (user_agent.startswith(u"Samsung") or
-              user_agent.startswith(u"SPH") or
-              user_agent.startswith(u"SGH")):
-            tolerance = first_space(user_agent)
-        elif user_agent.startswith(u"SAMSUNG/"):
-            tolerance = second_slash(user_agent)
-        elif u"Samsung/SGH-L870" in user_agent:
-            tolerance = iol(user_agent, u"/", 5)
+        for sams in self.SAMSUNGS:
+            if sams in user_agent:
+                tol1 = iol(user_agent, u"/", start_index=user_agent.index(sams))
+                tol2 = iol(user_agent, u" ", start_index=user_agent.index(sams))
+                tolerance = tol1 if tol1 < tol2 else tol2
+                break
         else:
             tolerance = len(user_agent)
+
         match = self.ris_matcher(user_agent, tolerance)
         #print "SamsungMatcher %s -> f_m_ua -> %s" % (user_agent, match)
         return match
@@ -713,7 +711,8 @@ class WindowsCEMatcher(AbstractMatcher):
 
     def can_handle(self, user_agent):
         return (u"Mozilla/" in user_agent and (u"Windows CE" in user_agent or
-                                               u"WindowsCE" in user_agent))
+                                               u"WindowsCE" in user_agent or
+                                               u"ZuneWP7" in user_agent))
 
     def find_matching_ua(self, user_agent):
         match = self.ld_matcher(user_agent, self.WINDOWS_CE_TOLERANCE)
@@ -725,7 +724,7 @@ class WindowsCEMatcher(AbstractMatcher):
 
 handlers = [NokiaMatcher(),
             LGUPLUSMatcher(),
-            AndroidMatcher(),
+            AndroidMatcher(normalizers.android),
             SonyEricssonMatcher(),
             MotorolaMatcher(),
             BlackberryMatcher(),
